function [x,E,C]=ea(A,b) % enclosurealgorithm % left as independent function as previously % uses vermidrad
% ! to be p-coded
% former function [xl,xu,flag,xli,xui,R,M]=ea(Al,Au,bl,bu,type,R,M) % enclosurealgorithm
% MATLAB function of 2005-11-03 translated into INTLAB on 2006-02-15
% started again on 2008-12-04, brought into form of VERSOFT files
% 2008-12-05 output variables E, C introduced, bounds xl, xu revived (put into C together with flag, R and M)
% 2008-12-11 final version, p-coded
% x is the enclosure computed
% if x is assigned, then so are C.xl, C.xu
% x may be assigned whereas R, M may be NaN's
gr=getround;
setround(0);
b=b(:); 
[m1,n]=size(A); 
% defaults
x=repmat(intval(NaN),n,1);
E.error='ea: none';
E.where='NaN';
E.value='NaN';
C.xl=repmat(intval(NaN),n,1);
C.xu=repmat(intval(NaN),n,1);
C.flag='failure to produce verified output';
C.R=repmat(intval(NaN),n,n);
C.M=repmat(intval(NaN),n,n);
% data check
if ~(m1==n&&length(b)==n)
    E.error='ea: sizes do not match';
    setround(gr); return
end
xr=verifylss(A,b); % verifylss enclosure
% case of noninterval data
if ~isintval(A)&&~isintval(b)&&~isnan(xr.inf(1))
    x=xr;
    C.flag='verified enclosure computed';
    setround(gr); return
end
% midpoints and radii
I=eye(n,n); I=infsup(I,I);
[ac,Delta]=vermidrad(A); % Ac, Delta are intval quantities
[bc,delta]=vermidrad(b); % bc, delta are intval quantities
% two inverses
R=inv(ac);                                                     
if isnan(R.inf(1,1)) % R not computed
    if ~isnan(xr.inf(1))
        x=xr; % verifylss enclosure outputted if HBR enclosure not found
    end
    E.error='ea: R not computed, verifylss enclosure outputted if it exists';
    setround(gr); return
end
% R computed
C.R=R; % output of inverse for possible further use
G=I-abs(R)*Delta;
M=inv(G);                                         
if isnan(M.inf(1,1)) % M not computed
    if ~isnan(xr.inf(1))
        x=xr; % verifylss enclosure outputted if HBR enclosure not found
    end
    E.error='ea: M not computed, verifylss enclosure outputted if it exists';
    setround(gr); return
end
% M computed
C.M=M; % output of inverse for possible further use
if ~all(all(M.inf>=zeros(n,n))) % not verified strongly regular
    if ~isnan(xr.inf(1))
        x=xr; % verifylss enclosure outputted if HBR enclosure not found
    end
    E.error='ea: interval matrix not verified strongly regular (M not verified nonnegative), verifylss enclosure outputted if it exists';
    [i1,j1]=find(M.inf<0,1); % M.inf(i1,j1)<0
    E.where=[i1 j1];
    E.value=M(i1,j1); % M.inf(i1,j1)<0; the whole M is in C.M
    setround(gr); return
end
% verified strongly regular
xc=verifylss(ac,bc); % midpoint solution 
if isnan(xc.inf(1)) % xc not computed
    if ~isnan(xr.inf(1))
        x=xr; % verifylss enclosure outputted if HBR enclosure not found
    end
    E.error='ea: xc not computed, verifylss enclosure outputted if it exists';
    setround(gr); return
end
% xc computed
% xa=M*(abs(xc)+abs(R)*delta); % former formula using the inverse
xa=verifylss(G,abs(xc)+abs(R)*delta); % solving system instead of employing inverse, for better accuracy
if isnan(xa.inf(1)) % xa not computed
    if ~isnan(xr.inf(1))
        x=xr; % verifylss enclosure outputted if HBR enclosure not found
    end
    E.error='ea: xa not computed, verifylss enclosure outputted if it exists';
    setround(gr); return
end
% xa computed
% next lines: computing the HBR enclosure
xlt=repmat(intval(0),n,1); % preallocation
xut=repmat(intval(0),n,1); % preallocation
% computing xlt
for i=1:n
    if xc.sup(i)<=0
        xlt(i)=-xa(i);
    else
        % xlt=-xa+diag(M).*(xc+abs(xc));
        xlt(i)=-xa(i)+M(i,i)*(xc(i)+abs(xc(i)));
    end
end
% computing xut
for i=1:n
    if xc.inf(i)>=0
        xut(i)=xa(i);
    else
        % xut= xa+diag(M).*(xc-abs(xc));
        xut(i)= xa(i)+M(i,i)*(xc(i)-abs(xc(i)));
    end
end
% HBR enclosure
nu=diag(2*M-I); % all(nu>=1)==1 in infinite precision
if any(nu<0.5) % to be sure
    if ~isnan(xr.inf(1))
        x=xr; % verifylss enclosure outputted if HBR enclosure not found
    end
    E.error='ea: any(nu<0.5)==1, HBR enclosure not computed, verifylss enclosure outputted if it exists';
    setround(gr); return
end
aux=xlt./nu;
xl=min(xlt.inf,aux.inf); % lower bound of the HBR enclosure computed
aux=xut./nu;
xu=max(xut.sup,aux.sup); % upper bound of the HBR enclosure computed
x=infsup(xl,xu); % HBR enclosure computed
if ~isnan(xr.inf(1)) 
    x=intersect(x,xr); % intersection with Rump's enclosure; hence the resulting x never worse than the verifylss enclosure
end
C.flag='verified enclosure of the interval hull computed, bounds on bounds computed';
% computation of inner bounds (to be stored in C.xl, C.xu) starts here
% formulae from 2006 remain unchanged
aux=infsup(xc.inf,xc.inf).*infsup(xc.sup,xc.sup);
if ~all(aux.inf>0) % sign of the midpoint solution could not be determined
    E.error='ea: sign of xc could not be computed';
    E.value=aux;
    C.xl=infsup(xl,xc.sup);
    C.xu=infsup(xc.inf,xu);
    setround(gr); return
else % computation of the inner bounds according to the 2000 formulae as reformulated in 2005 and recasted in 2006
    xl=infsup(xl,xl); % intval quantity
    xu=infsup(xu,xu); % intval quantity
    xil=abs(xl)+xl-xc-abs(xc);
    xiu=abs(xu)-xu+xc-abs(xc);
    zc=ones(n,1);
    for j=1:n
        if xc.inf(j)<0, zc(j)=-1; end % sign vector of xc determined
    end
    a=Delta*xa+delta;
    aR=abs(R);
    Mxil=Delta*M*diag(xil)+a*ones(1,n);
    Mxiu=Delta*M*diag(xiu)+a*ones(1,n);
    Mzc=inv(I-abs(R*diag(zc)*Delta));
    if isnan(Mzc.inf(1,1)) % inverse not computed
        E.error='ea: inverse of I-abs(R*diag(zc)*Delta) could not be computed';
        C.xl=infsup(xl,xc.sup); 
        C.xu=infsup(xc.inf,xu); 
        setround(gr); return   
    end
    Lzcm=diag(zc)*R*diag(zc);
    Lzc=Lzcm-aR;
    dl=repmat(intval(0),1,n); % preallocation
    du=repmat(intval(0),1,n);
    for i=1:n
        z=zc; 
        z(i)=-1; 
        if z(i)==zc(i) % z=zc
            dl(i)=Mzc(i,:)*abs(Lzc*Mxil(:,i));
        else 
            L=Lzcm; L(i,:)=-L(i,:); L(:,i)=-L(:,i); L=L-aR;
            dl(i)=M(i,:)*abs(L*Mxil(:,i));
        end
        z(i)=1;  
        if z(i)==zc(i) % z=zc
            du(i)=Mzc(i,:)*abs(Lzc*Mxiu(:,i));
        else 
            L=Lzcm; L(i,:)=-L(i,:); L(:,i)=-L(:,i); L=L-aR;
            du(i)=M(i,:)*abs(L*Mxiu(:,i));
        end
    end
    xli=xl+dl'; 
    xui=xu-du'; 
    xl=infsup(xl.inf,xli.sup);
    xu=infsup(xui.inf,xu.sup);
    C.xl=xl;
    C.xu=xu;
end
setround(gr);
% end of ea
%
%%%%%%%%%%%%%%%%%%%%%%%
% Subfunction vermidrad
%%%%%%%%%%%%%%%%%%%%%%%
%
function [Ac,Delta]=vermidrad(A)
% computes verified midpoint and radius of A
% Ac, Delta are intval quantities
if ~isintval(A)
    Ac=infsup(A,A);
    Delta=infsup(zeros(size(A)),zeros(size(A)));
else
    Al=infsup(A.inf,A.inf);
    Au=infsup(A.sup,A.sup);
    Ac=   (Al+Au)/2;        
    Delta=(Au-Al)/2;        
end
% end of vermidrad
%