function [x,E,C]=jz(varargin) % uses versolver (called 1+p+2*n times)
%
%    To be p-coded!! Built-in function.
%
%    parameter ch in versolver to be adjusted manually (3rd line of jz after help)
%
%    order of varargin{...}: A0, A1, ..., Ap, b0, b1, ..., bp, t (nargin=2*(p+1)+1)
%    Ah, bh at places h+1, p+2+h 
%    derivation of the basic equation: see verinthulldep-support.m
%    number of calls of versolver: 1+p+2*n
%    C has fields C.xenc, C.DxDt, C.D
%
%    History
%
%    2008-11-16   started (versolver created; up to construction of D; called verintervalhulldep)
%    2008-11-17   upper and lower loops added, input of a cell array enabled, 
%                 renamed as jz (2008-11-17 Er JZ upw), 
%                 called by verhullpar; working version
%    2008-11-19   intval bj allowed because of verhullpattern
%    2008-11-20   input choice parameter ch introduced into versolver, B omitted,
%                 output structured array C introduced, ch=2 as default
%    2008-12-04   ch=12 as default; nothing else except comments changed,
%                 and ch added into all calls of versolver where it has not been yet
%    2008-12-18   verifylss replaced by new ea in versolver
%
gr=getround;
setround(0);
ch=12; % default setting for versolver; 1=ea; 2=verintervalhull; 12=first 1, if NaN then 2; 21=first 2, if NaN then 1 (adjust also in versolver)
x=infsup(NaN,NaN); % default % should be nx1; n not yet determined
E.error='jz: none';
E.where='NaN';
E.value='NaN';
C.xenc=repmat(intval(NaN),1,1); % should be nx1; n not yet determined
C.DxDt=repmat(intval(NaN),1,1); % should be nxp; n, p not yet determined
C.D=repmat(NaN,1,1); % should be nxp; n, p not yet determined
% case of cell array as a single input variable (to be used by function handling special classes of parametric matrices)
if nargin==1 % one input variable; should be a cell array
    obj=varargin{1};
    if ~isa(obj,'cell')
        E.error='jz: the single input variable is not a cell';
        setround(gr); return
    end
    % obj is a cell array
    q=length(obj);
    for j=1:q % copy of obj into varargin 
        varargin{j}=obj{j};
    end
    nnargin=q; % nnargin is the length of obj
else
    nnargin=nargin; % case of nargin>1
end
if mod(nnargin-1,2)~=0 % nnargin-1 not even
    E.error='jz: improper number of input arguments';
    setround(gr); return
end
% nnargin-1 even
p=round((nnargin-1)/2)-1; % number of parameters % nnargin=2*(p+1)+1
if p<=0
    E.error='jz: improper number of input arguments';
    setround(gr); return
end
% checking input matrices
A0=varargin{1};
[m,n]=size(A0);
for j=1:p+1 % matrices Aj
    Aj=varargin{j};
    if ~(isequal(size(Aj),[n,n])&&isreal(Aj)&&~isintval(Aj))
        E.error='jz: input matrix of improper size or not real or of type intval';
        E.where=j;
        E.value=Aj;
        setround(gr); return
    end
end
% checking input vectors
for j=p+2:2*(p+1) % vectors bj
    bj=varargin{j};
    bj=bj(:);
    %if ~(isequal(size(bj),[n,1])&&isreal(bj)&&~isintval(bj))
    if ~(isequal(size(bj),[n,1])&&isreal(bj)) % intval bj allowed because of verhullpattern
        %E.error='jz: input vector of improper size or not real or of type intval';
        E.error='jz: input vector of improper size or not real';
        E.where=j;
        E.value=bj;
        setround(gr); return
    end
end
% checking the parameter vector
t=varargin{2*(p+1)+1}; % parameter vector
t=t(:);
if ~(isequal(size(t),[p,1])&&isreal(t)&&isintval(t))
    E.error='jz: parameter vector of improper size or not real or not of type intval';
    E.where=2*(p+1)+1;
    E.value=t;
    setround(gr); return
end
if nargout>3
    E.error='jz: too many output variables';
    setround(gr); return
end
x=repmat(infsup(NaN,NaN),n,1); % default of proper size
% end of data checking
% outer interval matrix
A=varargin{1};
A=intval(A);
for j=2:p+1
    A=A+t(j-1)*varargin{j};
end
% outer interval vector
b=varargin{p+2};
b=intval(b);
for j=p+3:2*(p+1)
    b=b+t(j-p-2)*varargin{j};
end
% solution of the outer system % and, possibly, inverse of the outer matrix
[xenc,Eversolver]=versolver(A,b,ch);
if isnan(xenc.inf(1))
    E=Eversolver;
    E.error=['jz:' E.error];
    % E.where='computation of solution of the outer system or inverse of the outer matrix';
    E.where='computation of solution of the outer system';
    E.value=NaN;
    setround(gr); return
end
C.xenc=xenc; % for output
% xenc, B computed: xenc enclosure of solution of the outer system, B enclosure of the inverse of the outer marix
% construction of matrices DxDt and D
DxDt=repmat(intval(0),n,p); % preallocation % interval matrix
for h=1:p
    % A(t)*(dx/dt_h) = bh - Ah*x % basic equation
    % DxDt(:,h)=B*(varargin{p+2+h}-varargin{h+1}*xenc); % old version: system not solved, inverse used instead (faster but enclosure wider)
    dxdth=versolver(A,varargin{p+2+h}-varargin{h+1}*xenc,ch); % in this version B not needed % 2008-11-20 B omitted at all % ch added 2008-12-04
    if ~isnan(dxdth.inf(1))
        DxDt(:,h)=dxdth;
    % else DxDt(:,h) remains a column of interval zeros
    end
end
C.DxDt=DxDt; % for output
% sign matrix
D=zeros(n,p); % preallocation % real matrix
for i=1:n
    for h=1:p
        if DxDt.inf(i,h)>0 % formerly >=0; changed because of initial zeros in DxDt
            D(i,h)=1;
        end
        if DxDt.sup(i,h)<0 % formerly <=0; changed because of initial zeros in DxDt
            D(i,h)=-1;
        end
    end
end
C.D=D; % for output
% computation of the bounds
xxx=xenc; % default % final xxx will satisfy in(xxx,xenc)==1
% upper bounds
for i=1:n
    if ~isequal(D(i,:),zeros(1,p)) % at least one entry of D(i,:) equal to 1 or -1
        tt=t; % copy of the parameter vector (column)
        % update of tt along D(i,:)
        for h=1:p
            if D(i,h)== 1 % increasing
                tt(h)=intval(tt.sup(h));
            end
            if D(i,h)==-1 % decreasing
                tt(h)=intval(tt.inf(h));
            end
        end
        % updated interval matrix (along tt)
        AA=varargin{1};
        AA=intval(AA);
        for j=2:p+1
            AA=AA+tt(j-1)*varargin{j};
        end
        % updated interval vector (along tt)
        bb=varargin{p+2};
        bb=intval(bb);
        for j=p+3:2*(p+1)
            bb=bb+tt(j-p-2)*varargin{j};
        end
        % solution of the updated system
        xx=versolver(AA,bb,ch);
        if ~isnan(xx.inf(1)) % xx computed
            %xxx.sup(i)=min(xx.sup(i),xxx.sup(i));
            xxx(i)=infsup(xxx.inf(i),min(xx.sup(i),xxx.sup(i)));
        %else xxx.sup(i) remains xenc.sup(i)
        end
        % i-th upper bound computed
    %else % D(i,:) is zero % xxx.sup(i) remains xenc.sup(i)
    end
end % of upper bounds
% lower bounds
for i=1:n
    if ~isequal(D(i,:),zeros(1,p)) % at least one entry of D(i,:) equal to 1 or -1
        tt=t; % copy of the parameter vector (column)
        % update of tt along D(i,:)
        for h=1:p
            if D(i,h)== 1 % increasing
                tt(h)=intval(tt.inf(h));
            end
            if D(i,h)==-1 % decreasing
                tt(h)=intval(tt.sup(h));
            end
        end
        % updated interval matrix (along tt)
        AA=varargin{1};
        AA=intval(AA);
        for j=2:p+1
            AA=AA+tt(j-1)*varargin{j};
        end
        % updated interval vector (along tt)
        bb=varargin{p+2};
        bb=intval(bb);
        for j=p+3:2*(p+1)
            bb=bb+tt(j-p-2)*varargin{j};
        end
        % solution of the updated system
        xx=versolver(AA,bb,ch);
        if ~isnan(xx.inf(1)) % xx computed
            %xxx.inf(i)=max(xx.inf(i),xxx.inf(i));
            xxx(i)=infsup(max(xx.inf(i),xxx.inf(i)),xxx.sup(i));
        %else xxx.inf(i) remains xenc.inf(i)
        end
        % i-th lower bound computed
    %else % D(i,:) is zero % xxx.inf(i) remains xenc.inf(i)
    end
end % of lower bounds
% bounds computed, result in xxx
x=xxx;
setround(gr);
% end of jz
%
%%%%%%%%%%%%%%%%%%%%%%%
% Subfunction versolver
%%%%%%%%%%%%%%%%%%%%%%%
%
function [x,E]=versolver(A,b,ch)
% for interval or real data A,b
% computes an enclosure x of the solution set of A*x=b
%%% and an enclosure B of the inverse of A
% or yields NaN's
% formerly, verifylss preferred to verintervalhull; later, chosen by the input parameter ch
%
%%% use x=versolver(A,b) if only the system is to be solved (then B internally not computed)
%
% 2008-11-16   created
% 2008-11-17   embedded into verhullpar
% 2008-11-20   computation of inverse suppressed (no more needed in the main file), 
%              input parameter ch (choice of method) introduced; default ch=2
% 2008-12-04   default ch=1
% 2008-12-18   verifylss replaced by new ea
%
gr=getround;
setround(0);
if nargin<3
    ch=1; % default choice % 1=ea; 2=verintervalhull; 12=first 1, if NaN then 2; 21=first 2, if NaN then 1 
end
b=b(:);
[m,n]=size(A);
% defaults
x=repmat(infsup(NaN,NaN),n,1);
% B=repmat(infsup(NaN,NaN),m,n);
E.error='versolver: none';
E.where='NaN';
E.value='NaN';
% data check
if ~(m==n&&length(b)==m)
    E.error='versolver: sizes do not match';
    setround(gr); return
end
% making data of type intval
if ~isintval(A)
    A=intval(A);
end
if ~isintval(b)
    b=intval(b);
end
% computation
switch ch
    case  1
        xx=ea(A,b);
    case  2
        xx=verintervalhull(A,b);
    case 12
        xx=ea(A,b);
        if isnan(xx.inf(1)) % failed
            xx=verintervalhull(A,b);
        end
    case 21
        xx=verintervalhull(A,b);
        if isnan(xx.inf(1)) % failed
            xx=ea(A,b);
        end
    otherwise
        % no other options as yet
end
if isnan(xx.inf(1)) % failed
    E.error=['versolver: verified solution of the system not found (ch=' int2str(ch) ')'];
    setround(gr); return
end
x=xx;
setround(gr);
% end of versolver
%
